<?php

/**
 * Copyright 2017-2024 IQRF Tech s.r.o.
 * Copyright 2019-2024 MICRORISC s.r.o.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
declare(strict_types = 1);

namespace App\ApiModule\Version0\Controllers\Security;

use Apitte\Core\Annotation\Controller\Method;
use Apitte\Core\Annotation\Controller\OpenApi;
use Apitte\Core\Annotation\Controller\Path;
use Apitte\Core\Annotation\Controller\Tag;
use Apitte\Core\Http\ApiRequest;
use Apitte\Core\Http\ApiResponse;
use App\ApiModule\Version0\Controllers\Gateway\BaseGatewayController;
use App\ApiModule\Version0\Models\ControllerValidators;

/**
 * Gateway password controller
 */
#[Path('/password')]
#[Tag('Security - Shell user password')]
class PasswordController extends BaseGatewayController {

	/**
	 * Constructor
	 * @param ShellUserController $newController New shell user controller
	 * @param ControllerValidators $validators Controller validators
	 */
	public function __construct(
		private readonly ShellUserController $newController,
		ControllerValidators $validators,
	) {
		parent::__construct($validators);
	}

	#[Path('/')]
	#[Method('PUT')]
	#[OpenApi('
		summary: Updates default gateway user password
		deprecated: true
		description: "Deprecated in favor of the new shell user controller, use `PUT` `/security/shellUser/password` instead. Will be removed in the version 3.1.0."
		requestBody:
			required: true
			content:
				application/json:
					schema:
						$ref: \'#/components/schemas/GatewayPassword\'
		responses:
			\'200\':
				description: Success
			\'400\':
				$ref: \'#/components/responses/BadRequest\'
			\'403\':
				$ref: \'#/components/responses/Forbidden\'
			\'500\':
				$ref: \'#/components/responses/ServerError\'
	')]
	public function setPassword(ApiRequest $request, ApiResponse $response): ApiResponse {
		return $this->newController->setPassword($request, $response);
	}

}
