<?php

declare(strict_types = 1);

/**
 * Copyright 2017-2024 IQRF Tech s.r.o.
 * Copyright 2019-2024 MICRORISC s.r.o.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

namespace App\Models\Database\Enums;

use App\Exceptions\InvalidUserRoleException;
use ValueError;

/**
 * User roles enum
 */
enum UserRole: string {

	/// Admin role
	case Admin = 'admin';

	/// Normal role
	case Normal = 'normal';


	/// Basic admin role
	case BasicAdmin = 'basicadmin';

	/// Basic role
	case Basic = 'basic';

	/// Default role
	final public const Default = self::Normal;

	/**
	 * Returns user role from string value
	 * @param string $value User role string value
	 * @return self User role
	 * @throws InvalidUserRoleException
	 */
	public static function fromString(string $value): self {
		try {
			return self::from($value);
		} catch (ValueError $e) {
			throw new InvalidUserRoleException('Invalid role: "' . $value . '"', previous: $e);
		}
	}

}
