<?php
/**
 * Copyright 2018 IQRF Tech s.r.o.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
declare(strict_types = 1);

namespace Iqrf\IdeMacros;

/**
 * IQRF IDE Macro's group
 */
class Group {

	/**
	 * @var string Group name
	 */
	public $name;

	/**
	 * @var int Group ID
	 */
	public $id;

	/**
	 * @var bool Is group enabled?
	 */
	public $enabled;

	/**
	 * @var Macro[] Array of macros
	 */
	public $macros;

	/**
	 * Group object constructor
	 * @internal
	 * @param string[] $array Macro group's data in an array
	 * @param MacroFileParser $file IQRF IDE Macro file
	 */
	public function __construct(array $array, MacroFileParser $file) {
		$this->name = $array[0];
		$this->enabled = $array[1] === 'True';
		$this->id = intval($array[2]);
		$macros = array_chunk(array_slice($array, 3), $file->getMacroSize());
		foreach ($macros as $macro) {
			$this->macros[] = new Macro($macro, $file);
		}
	}

	/**
	 * Returns the group in an array
	 * @return mixed[] Group in an array
	 */
	public function toArray(): array {
		$array = get_object_vars($this);
		foreach ($this->macros as $id => $macro) {
			assert($macro instanceof Macro);
			$array['macros'][$id] = $macro->toArray();
		}
		return $array;
	}

}
